function [x, P] = align_svd(Sr, So, x)
%
% Tim Bailey 2004

NN = 20;    % number near-neighbours
Nopt = 25;  % number optimise steps
Ns = 36;    % number PDF samples
Nloop = 20; % number of retries if P is not positive-definite.
Ps = [.1; .1; pi/180];
npsd = 0;

% Get maximum likelihood alignment (m), and its correlation weight (K)
[m, K] = sog_optimise(Sr, So, x, Nopt, NN); 

% This next part is performed in a loop so as to reject non-PSD solutions
while 1
    
    % Generate Ns samples in region of m
    x = [];    
    while size(x,2) < Ns
        x = [x sample_geometric(m, Ps)];
%        x = [x sample_cube(m, Ps)];
    end
    x = x(:,1:Ns);    
    
    % Compute correlation value for each sample
    for i=1:Ns
        C(i) = sog_correlate(Sr, So, x(:,i), NN);
    end
 
    if K==0 | any(C==0)
        P=0;
        return;
    end
    
    % Compute Gaussian
    P = gauss_variance_svd(m,K, x,C);
    x = m;
    
    % Check result is positive definite
    [R,p] = chol(P);
    if p==0
        break
    else
        npsd = npsd+1;
        if npsd > Nloop, P=0; return, end
    end
end

%
%

function s = sample_geometric(x, s)

sx = x(1) + [0 0 0 1 1 1] * s(1) + randn(1)*s(1);
sy = x(2) + [0 1 1 0 0 1] * s(2) + randn(1)*s(2);
sp = x(3) + [1 1 0 1 0 0] * s(3) + randn(1)*s(3);
s = [sx; sy; sp];

%
%

function s = sample_cube(x, s)

sx = x(1) + [-1 -1 -1 -1  1  1 1  1] * 0.5 * s(1) + randn(1)*s(1);
sy = x(2) + [-1 -1  1  1 -1 -1 1  1] * 0.5 * s(2) + randn(1)*s(2);
sp = x(3) + [ 1 -1  1 -1  1 -1 1 -1] * 0.5 * s(3) + randn(1)*s(3);
s = [sx; sy; sp];

%
%

function s = sample_random(x, s)

sx = x(1) + randn(1,N) * s(1);
sy = x(2) + randn(1,N) * s(2);
sp = x(3) + randn(1,N) * s(3);
s = [sx; sy; sp];
        