function demo_gauss_svd()
% This demo shows how to estimate a Gaussian (mean and variance) for scan-correlation
% by fitting a Gaussian function to samples from the correlation function. The SVD 
% is used to give a least-squares fit.
%
% Running this demo several times will show to things: 
%   (1) How very over-conservative the sog_correlation algorithm is, and
%   (2) How variable the align_svd algorithm is in fitting a Gaussian to the correlation function.
%
% Tim Bailey 2004.

load data % load scans (r1, r2) and an initial relative pose estimate (rel)

MAXR = 7.8;                     % this laser data times-out at ~7.9 m
R = [0.05^2 0; 0 (pi/360)^2];   % range-bearing covariance matrix
phi = -pi/2:(pi/360):pi/2;      % bearing angles for each laser scan

Sr = make_gaussian_sum(r1, phi, R, MAXR);
So = make_gaussian_sum(r2, phi, R, MAXR);

[x, P] = align_svd(Sr, So, rel);

% plots
x1 = [Sr.x];
x2 = transform_to_global([So.x], x);
pc= sigma_ellipse(x(1:2), P(1:2,1:2), 2); 
figure
plot(x1(1,:),x1(2,:),'.', x2(1,:),x2(2,:),'.', x(1),x(2),'r*', pc(1,:), pc(2,:), 'r');
axis equal
grid
