function P = gauss_variance_svd(m,K, x,C)
%function P = gauss_variance_svd(m,K, x,C)
%
% m - max likelihood location
% K - max likelihood value
% x - set of N locations
% C - N likelihoods (one for each location)
%
% Determines the covariance of a PDF from a set of point evaluations. It assumes
% the PDF is Gaussian, and computes the true covariance if this is the case.
% The covariance is computed on N (>> 6) samples to permit SVD least-squares.
%
% Tim Bailey 2004.

x = x - repmat(m, 1, size(x,2));

A = [ x(1,:).^2;
    2*x(1,:).*x(2,:);
    2*x(1,:).*x(3,:);
    x(2,:).^2;
    2*x(2,:).*x(3,:);
    x(3,:).^2 ]';

b = -2 * (log(C') - log(K));

[U,S,V] = svd(A);
Si = diag(1./diag(S));
Si = [Si zeros(6, size(x,2)-6)];
a = V*Si*U'*b;

P = [a(1) a(2) a(3);
     a(2) a(4) a(5);
     a(3) a(5) a(6)];
 
P = inv(P);
