function demo_importance_gauss()
% This demo shows how to estimate a Gaussian (mean and variance) for scan-correlation
% using importance sampling. The results of this method are very sensitive to the proposal
% distribution and, essentially, this is not a very reliable method if the correlation
% function is very non-Gaussian.
%
% Tim Bailey 2004.

load data % load scans (r1, r2) and an initial relative pose estimate (rel)

MAXR = 7.8;                     % this laser data times-out at ~7.9 m
R = [0.05^2 0; 0 (pi/360)^2];   % range-bearing covariance matrix
phi = -pi/2:(pi/360):pi/2;      % bearing angles for each laser scan

N = 1000;                   % number of importance samples
P = [0.2^2 0 0;             % proposal distribution variance
     0 0.2^2 0;
     0 0 (0.5*pi/180)^2];

Sr = make_gaussian_sum(r1, phi, R, MAXR);
So = make_gaussian_sum(r2, phi, R, MAXR);

[x,P] = importance_gaussian(Sr,So,rel, P,N);

% plots
x1 = [Sr.x];
x2 = transform_to_global([So.x], x);
pc= sigma_ellipse(x(1:2), P(1:2,1:2), 2); 
figure
plot(x1(1,:),x1(2,:),'.', x2(1,:),x2(2,:),'.', x(1),x(2),'r*', pc(1,:), pc(2,:), 'r');
axis equal
grid
