function [x,P] = importance_gaussian(Sr,So,x0, P,N)
%
% Tim Bailey 2004.

xs = multivariate_gauss(x0,P,N);    % proposal distribution
 
wg = evaluate_gauss(xs, x0,P);      % proposal weight
wc = evaluate_correlate(xs, Sr,So); % actual weight
w = wc./wg;                         % importance weight

xs = resample(xs,w);                % create equi-weight samples
[x,P] = sample_mean(xs);            % compute mean and variance of samples

%
%

function w = evaluate_gauss(xi,x,P)
% Compute the density of G(x,P) at points xi
N = length(x);
Pinv = inv(P);

denom = sqrt((2*pi)^N * det(P));

len = size(xi,2);
w = zeros(1,len);
for i=1:len
    v = xi(:,i) - x;
    num = exp(-0.5 * v' * Pinv * v);
    w(i) = num/denom;
end

%
%

function w = evaluate_correlate(x, Sr,So)
% Compute the scan-correlation weights for relative poses x
KNN = 20;
len = size(x,2);
w = zeros(1,len);
for i=1:len
    w(i) = sog_correlate(Sr, So, x(:,i), KNN);
end

%
%

function x = resample(x,w)
% Resample particles
[keep, Neff] = stratified_resample(w);
x = x(:,keep);
