function SOG = make_gaussian_sum(r, phi, R, max_r)
%
% Inputs:
%   r - set of 361 laser range measurements
%   phi - associated bearing angles (eg, phi = -pi/2:(pi/360):pi/2)
%   R - range-bearing covariance (eg, R = [.1^2 0; 0 (1*pi/180)^2])
%   max_r - maximum accepted range 
%
% Output:
%   SOG - array of structures for Gaussian Sum

if length(r) ~= length(phi), warning('number ranges does not match number angles'), end

ii = find(r < max_r);
r = r(ii);
phi = phi(ii);

for i=1:length(r)
    ri = r(i);
    ci = cos(phi(i));
    si = sin(phi(i));
    
    SOG(i).x = [ri*ci; ri*si];

    J = [ci -ri*si; si ri*ci];
    SOG(i).P = J * R * J';
    
    SOG(i).w = 2*pi * sqrt(det(SOG(i).P));
end
