function [xm,P]= sample_mean(x)
%function [xm,P]= sample_mean(x)
% INPUT: set of samples, where each sample is a vector [x1;...;xi;...]
% OUTPUTS: sample mean, and sample covariance matrix
%
% Convert a set of samples to a mean vector and covariance matrix.
% Results of this function are (should be) equivalent to calling
% xm= mean(x'); and P= cov(x'); from the MatLab library.
%
% Tim Bailey 2002.

n= size(x,2); % number of samples
len= size(x,1); % length of state vector
xm= zeros(len,1);
P= zeros(len,len);

% calculate all means sum(xi)
for i=1:len
    xm(i)= sum(x(i,:))/n;
end

% return if only one sample
if n==1, return, end

% calculate all cross correlations sum(xi*xj)
for i=1:len
    for j=i:len
        P(i,j)= (sum(x(i,:).*x(j,:)) - n*xm(i)*xm(j)) / (n-1);
        P(j,i)= P(i,j);
    end
end
