function demo_icp()
% This demo uses ICP to perform scan-match based dead-reckoning. It plots a
% map based on these dead-reckoning results.
%
% Tim Bailey 2004.

load laserdata % logged laser range scans from indoor robot
figure, axis equal, grid, hold on

% Configurable values
INTERP = 1; % 0 - simple ICP, 1 - ICP with interpolation
NIT = 10;   % number of ICP iterations for each scan-match
GATE1 = 0.5; % 1st data association gate for each point in scan
GATE2 = 0.05;% 2nd data association gate for each point in scan

% Constants
MAXR = 7.8; % this laser data times-out at ~7.9 m

% Initialise
x = [0;0;0];
delta = [0;0;0];
p1 = get_laser_points(laser(1,2:end), MAXR);

% Loop
for i=2:size(laser,1)
    p2 = get_laser_points(laser(i,2:end), MAXR);
    delta = icp(p1, p2, delta, GATE1, NIT, INTERP);
    delta = icp(p1, p2, delta, GATE2, NIT, INTERP);
    
    x = transform_to_global(delta, x);
    p1 = p2;

    % plots
    pg = transform_to_global(p2, x);
    plot(x(1),x(2), '+', pg(1,:), pg(2,:), '.')
    %drawnow
end

%
%

function x = get_laser_points(r, MAXR)
phi = -pi/2:(pi/360):pi/2;
ii = find(r < MAXR);
r = r(ii); phi = phi(ii);
x = [r.*cos(phi); r.*sin(phi)];
