function demo_icp_2()
% This demo performs ICP on each scan-pair and plots them together in a 
% common coordinate frame. It shows the alignment of points and the actual
% point-wise associations used to compute the ICP relative pose.
%
% Tim 2004.

load laserdata % logged laser range scans from indoor robot

% Configurable values
INTERP = 1; % 0 - simple ICP, 1 - ICP with interpolation
NIT = 20;   % number of ICP iterations for each scan-match
GATE1 = 0.5;   % 1st data association gate for each point in scan
GATE2 = 0.05;   % 2nd data association gate for each point in scan

% Constants
MAXR = 7.8; % this laser data times-out at ~7.9 m

% Setup plots
figure, axis equal, grid, hold on
h = plot(0,0,'.', 0,0,'.', 0,0,'r', 'erasemode', 'xor');

% Initialise
delta = [0;0;0];
p1 = get_laser_points(laser(1,2:end), MAXR);

% Loop
for i=2:size(laser,1)
    p2 = get_laser_points(laser(i,2:end), MAXR);
    [delta, a, b] = icp(p1, p2, delta, GATE1, NIT, INTERP);
    [delta, a, b] = icp(p1, p2, delta, GATE2, NIT, INTERP);

    pg = transform_to_global(p2, delta);
    pa = compute_associations(a, b, delta);
    set(h(1), 'xdata', p1(1,:), 'ydata', p1(2,:))
    set(h(2), 'xdata', pg(1,:), 'ydata', pg(2,:))
    set(h(3), 'xdata', pa(1,:), 'ydata', pa(2,:))
    pause
    
    p1 = p2;
end

%
%

function x = get_laser_points(r, MAXR)
phi = -pi/2:(pi/360):pi/2;
ii = find(r < MAXR);
r = r(ii); phi = phi(ii);
x = [r.*cos(phi); r.*sin(phi)];

%
%

function p = compute_associations(a, b, x)
b = transform_to_global(b, x);
p = [];
for i=1:size(a,2)
    p = [p a(:,i) b(:,i) [NaN;NaN]];
end

