function x = demo_icp_3(r1, r2, x0)
% This demo performs ICP on scans r1 and r2, given an initial guess x0 of the pose of r2 relative to r1.
% It shows the points of the two scans and the associations between them (indicated by red lines)
%
% Tim 2004.

% Configurable values
INTERP = 1; % 0 - simple ICP, 1 - ICP with interpolation
NIT = 20;   % number of ICP iterations for each scan-match
GATE1 = 0.5;    % 1st data association gate for each point in scan
GATE2 = 0.05;   % 2nd data association gate for each point in scan

% Constants
MAXR = 7.8; % this laser data times-out at ~7.9 m

% Compute
p1 = get_laser_points(r1, MAXR);
p2 = get_laser_points(r2, MAXR);
[x, a, b] = icp(p1, p2, x0, GATE1, NIT, INTERP);
[x, a, b] = icp(p1, p2, x,  GATE2, NIT, INTERP);

pg = transform_to_global(p2, x);
pa = make_association_lines(a, b, x);
figure, axis equal, grid, hold on
plot(p1(1,:),p1(2,:),'.', pg(1,:),pg(2,:),'.', pa(1,:),pa(2,:),'r');

%
%

function x = get_laser_points(r, MAXR)
phi = -pi/2:(pi/360):pi/2;
ii = find(r < MAXR);
r = r(ii); phi = phi(ii);
x = [r.*cos(phi); r.*sin(phi)];

%
%

function p = make_association_lines(a, b, x)
b = transform_to_global(b, x);
p = [];
for i=1:size(a,2)
    p = [p a(:,i) b(:,i) [NaN;NaN]];
end

