function demo_sog_optimise()
% This demo uses Sum of Gaussian (SoG) scan-matching to perform laser-based dead-reckoning. It plots a
% map based on these dead-reckoning results.
%
% Tim Bailey 2004.

load laserdata % logged laser range scans from indoor robot
figure, axis equal, grid, hold on

% Configurable values
Nit = 50; % number of optimisation iterations
knn = 15; % number of near-neighbours (0 means all neighbours)

% Constants
MAXR = 7.8;                     % this laser data times-out at ~7.9 m
R = [0.05^2 0; 0 (pi/360)^2];   % range-bearing covariance matrix
phi = -pi/2:(pi/360):pi/2;      % bearing angles for each laser scan

% Initialise
x = [0;0;0];
delta = [0;0;0];
S1 = make_gaussian_sum(laser(1,2:end), phi, R, MAXR); 

% Loop
for i=2:size(laser,1)
    S2 = make_gaussian_sum(laser(i,2:end), phi, R, MAXR); 
    delta = sog_optimise(S1, S2, delta, Nit, knn);
    S1 = S2;   

    % plots
    x = transform_to_global(delta, x);
    pg = transform_to_global([S2.x], x);
    plot(x(1),x(2), '+', pg(1,:), pg(2,:), '.')
    %drawnow
    disp(size(laser,1)-i)
end
