/*--------------------
 
	ai = nearest_neighbour(a,b,gate)

	INPUTS:
	a,b - two sets of points [x; y]
	gate - maximum permitted distance to a NN

	OUTPUTS:
	ai - index of nearest neighbour in a for each b

/*--------------------*/

#include "mex.h"
#include "../source/nn.hpp"

using namespace std;
using namespace Geom2D;

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
	// check arguments
    if (nrhs != 3 || nlhs > 1) 
		mexErrMsgTxt("Usage: ai = nearest_neighbour(a,b,gate)"); 
    if (mxGetM(prhs[0])!=2 || mxGetM(prhs[1])!=2)
		mexErrMsgTxt("Incorrect input matrix sizes.");

	// sizes
	const int Na = mxGetN(prhs[0]);
	const int Nb = mxGetN(prhs[1]);

	// get input parameters
	Point* p = reinterpret_cast<Point*>(mxGetPr(prhs[0])); 
	vector<Point> pa(p, p + Na);
	Point* pb = reinterpret_cast<Point*>(mxGetPr(prhs[1]));
	double gate = static_cast<double>(*mxGetPr(prhs[2]));

	// create output
	plhs[0] = mxCreateDoubleMatrix(1, Nb, mxREAL);
	double *idx = mxGetPr(plhs[0]);

	// algorithm
	SweepSearch nn(pa, gate);
	for (int i = 0; i < Nb; ++i) 
		idx[i] = nn.query(pb[i]) + 1;
}
