/*--------------------
 
	[an,d] = nearest_neighbour(a,b)

	INPUTS:
	a,b - two sets of points [x; y]

	OUTPUTS:
	ai - index of nearest neighbour in a for each b
	d - distance of each ai from b

/*--------------------*/

#include "mex.h"
#include "nn.h"

using namespace std;

#define PNTA	prhs[0]
#define PNTB	prhs[1]
#define NNA		plhs[0]
#define DIST	plhs[1]

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray*prhs[] )     
{
	// check arguments
    if (nrhs != 2 || nlhs>2) 
		mexErrMsgTxt("Usage: [an,d]= nearest_neighbour(a,b)"); 
    if (mxGetM(PNTA)!=2 || mxGetM(PNTB)!=2)
		mexErrMsgTxt("Error: Incorrect input matrix sizes.");

	// algorithm

	// create search data structure
	Point* p= reinterpret_cast<Point*>(mxGetPr(PNTA)); 
	vector<Point> pa(p, p+mxGetN(PNTA));
	sweep_search nn(pa);

	// create output
	int sizeb= mxGetN(PNTB);
	NNA = mxCreateDoubleMatrix(2, sizeb, mxREAL);
	DIST= mxCreateDoubleMatrix(1, sizeb, mxREAL);

	// get neighbours
	Point* pb= reinterpret_cast<Point*>(mxGetPr(PNTB));
	Point* pn= reinterpret_cast<Point*>(mxGetPr(NNA));
	double* d= mxGetPr(DIST);

	for (int i=0; i<sizeb; ++i) {
		int idx= nn.query(pb[i]);
		pn[i]= pa[idx];
		d[i]= dist(pb[i],pn[i]);
	}
}
