/* Iterated closest point (ICP) algorithm.
 * Tim Bailey 2004.
 */
#include "icp.hpp"
#include <cassert>

using namespace std;

namespace Geom2D {

ICP::ICP(const vector<Point>& reference, const double& gate) : 
ref(reference), nn(new SweepSearch(reference, gate))
{
}

Pose ICP::align(const vector<Point>& obs, const Pose& init, double gate, int nits, bool interp) 
{
	Pose pse = init; 
	double gate_sqr = sqr(gate); 
	int size_obs = obs.size();
	vector<int> index(2); // used if interp == true

	while (nits-- > 0) {
		Transform2D tr(pse);
		a.clear();
		b.clear();

		// For each point in obs, find its NN in ref
		for (int i = 0; i < size_obs; ++i) { 
			Point p = obs[i];
			tr.transform_to_global(p); // transform obs[i] to estimated ref coord-frame

			Point q;
			if (interp == false) { // simple ICP
				int idx = nn->query(p);
				if (idx == SweepSearch::NOT_FOUND)
					continue;

				q = ref[idx];
			} 
			else { // ICP with interpolation between 2 nearest points in ref
				(void) nn->query(p, index);
				assert(index.size() == 2);
				if (index[1] == SweepSearch::NOT_FOUND)
					continue;

				Line lne;
				lne.first  = ref[index[0]];
				lne.second = ref[index[1]];
				intersection_line_point(q, lne, p);
			}

			if (dist_sqr(p,q) < gate_sqr) { // check if NN is close enough 
				a.push_back(obs[i]);
				b.push_back(q);
			}
		}
		
		assert(a.size() > 2); // TODO: replace this assert with a proper status flag
		pse = compute_relative_pose(a, b); // new iteration result
	}

	return pse;
}

} // namespace Geom2D
