/////////////////////////////////////////////
//
// Downhill simplex (Nelder-Mead) algorithm. Adapted from Numerical Recipes
// in C, 1992.

#ifndef SIMPLEX_HEADER
#define SIMPLEX_HEADER

#include <vector>

typedef std::vector<double> val_type;

// function object interface definition
class simplex_functor {
public:
	virtual double operator () (const val_type& x) const = 0;
};

// simplex class
class simplex {
public:

	simplex(const val_type& start, const val_type& lamda, const double& ftol,
		const simplex_functor& f);

	void reset(const val_type& lamda);
	bool optimise(int num_iterations);

	int num_iterations() { return num_its; }
	const double& optimal_value() const { return f_best; }
	const val_type& optimal_state() const { return v_best; }

private:
	const double tol;
	const simplex_functor& func;

	std::vector<val_type> vertices;
	std::vector<double> f_vals;
	val_type v_sum;
	int ilo;
	int ihi;
	int ihi2;	
	int num_its;

	val_type v_best;
	double f_best;

	void make_vertices(const val_type& start, const val_type& lamda);
	void vertex_sum();
	void get_vertex_indices();
	double func_eval(double factor);
	void contract();
	void set_best();
};

#endif