//////////////////////////////
//

#include "likelihood_function.hpp"
#include "nn.hpp"
#include <vector>

namespace Geom2D {

struct SoG { // Sum of Gaussians
	Point x;
	Matrix P;
	double w;
};

class sog_correlate {
public:
	sog_correlate(const std::vector<SoG>& Sr, const std::vector<SoG>& So, 
		int knn=0, double prob_false_alarm=0., bool ignore_outlier=false); 
	~sog_correlate() { delete nn; }

	double get_weighting(const Pose& p) const;
	double get_weighting_variant(const Pose& p) const;
	std::vector<double>& get_variant_weights() { return wo; }

private:
	const std::vector<SoG>& Sr;
	const std::vector<SoG>& So;
	const double Pf;
	const bool IgnoreOutlier;

	mutable std::vector<int> idx;
	mutable std::vector<double> wo;
	std::vector<Point> pr;
	SweepSearch *nn;
};

class sog_optimise {
public:
	sog_optimise(const std::vector<SoG>& Sr, const std::vector<SoG>& So, int k=0);
	double optimise_pose(Pose &p, int num_its) const;

private:
	sog_correlate sc;
};

} // namespace Geom2D 
